package Test::Spec::RMock::AnyConstraint;

sub new  { bless {}, $_[0] }
sub call { 1 }

1;

__END__

=pod

=head1 NAME

Test::Spec::RMock::AnyConstraint

=head1 VERSION

version 0.005001

=head1 AUTHOR

Kjell-Magne Øierud <kjellm@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2012 by Kjell-Magne Øierud.

This is free software, licensed under:

  The MIT (X11) License

=cut
